from setuptools import setup, find_packages

setup(
    name="iflow-mcp_invoke-re-binja-lattice-mcp",
    version="0.1.0",
    packages=find_packages(),
    py_modules=["mcp_server", "mcp_server_test", "lattice_client"],
    install_requires=[
        "mcp>=1.6.0",
        "requests>=2.32.3",
    ],
    extras_require={
        "test": [
            "pytest>=7.4.0",
            "pytest-cov>=4.1.0",
        ],
    },
    entry_points={
        "console_scripts": [
            "binja-lattice-mcp=mcp_server_test:main",
        ],
    },
)